package Constraints;

use strict;
our (@ISA, @EXPORT);

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(CON CONdescription);


my %CON;
sub CON { return \%CON; }

my %CONdescription;
sub CONdescription { return \%CONdescription; }



#
#  Notes to constraint writers:
#
#  About the structure of constraints
#  ----------------------------------
#
#  1. Give your constraint a meaningful identifier.
#     Start with "F_"  for a faithfulness constraint
#                "M_"  for markedness
#                "Mz_" for markedness that starts with a "*", for alphabetic ordering purposes
#                "P_"  for PREC constraints
#     Use only letters, numbers and "_" in the identifiers.
#
#  2. Each constraint needs a description and a definition of a subroutine.
#     Don't forget a semicolon at the end of each. It's probably best to start
#     by copying and pasting a constraint that's closest to what to want to do. 
#
#  3. The first line of the subroutine returns the constraint name in HTML
#     in case the subroutine doesn't get any arguments.
#     For better compatibility with Praat, please use the 'small caps' style only around
#     the letters that actually need it.
#
#  4. For Faithfulness and Markedness constraints, CCamelOT gives you an input form and
#	an output form. You must return an integer that is the number of violations.
#     Markedness constraints should ignore the input.
#     Faithfulness constraints must return 0 if the input is missing - this is done in
#     syllabification, which must proceed without faithfulness violations.
#
#  5. PREC constraints are under construction
#
#
#  About the structure of linguistic forms
#  ---------------------------------------
#
#  1. Segments are found in two arrays, one that contains indices and one that contains
#     phoneme names. For instance, if your form is "ta:nki", the two arrays are "(1,2,3,4,5)"
#     and "('t','a','n','k','i')".
#
#     The "->indices()" and "->phonemes()" subroutines return a reference to the required
#     arrays. Use @{ ... } to get the array itself. Use ${ ... }[0] to get the first 
#     element of the array.
#     For instance, the phonemes of the inputs are in the array @{$inputForm->phonemes()}.
#     The first phoneme of the input is ${$inputForm->phonemes()}[0].
#
#     It's the program's job to keep these two arrays in sync. Since you are only going
#     to be reading them, not changing them, you don't have to worry about that.
#
#  2. A form's moras and syllables are accessed using the "->moras()" and "->syllables()"
#     methods. You will get a reference to an array of strings. Each string is one mora
#     or one syllable.
#     For a form like "ta:nki", the moras and syllables will look something like this:
#     Moras: ("1_mi2im","2_mi2i3im","3_mi5im")
#     Syllables: ("1_si1i2i3is","2_si4i5is")
#     As you can see, each unit begins with an identifier (index), followed by "_".
#     The indices of the segments that belong to each unit are surrounded by "i"'s
#     (for "index"). The unit's edges are marked by 'm' for mora and 's' for syllable.
#
#     Note that inputs will never have syllables, so "@{$inputForm->syllables()}" is 
#     always an empty array.
#
#     To find the phonemes that correspond to the indices in a certain mora or syllable,
#     use the "->findPhonemes()" method. Give it a mora or a syllable, and it returns
#     an array of phoneme names. Look at "Onset" or "NoCoda" for examples.
#
#  3. A form's root is accessed using the "->root()" subroutine, which returns a reference
#     to an array of indices. Look at AlignR(s,root) for an example.
#     Note that input forms can contain root segments and affixal segments.
#     Output forms can contain root segments, affix segments and epenthetic segments.
#
#  4. To access features, use the main::seg() subroutine. Give it a phoneme's name, 
#     such as "t", "glot", "schwa" (see the full list of phoneme names in Phoneme.pm).
#     Use the "->cons()" subroutine to the get the value of the feature consonantal.
#     See the full list of methods in Phoneme.pm
#
#  Debugging
#  ---------
#
#  The only way to test that your constraint actually does what you want is to 
#  use it in a derivation. But sometimes you aren't sure that you are getting
#  the values that you need inside the subroutine.
#
#  What you can do is return some arbitrary string instead of the number of violations.
#  To display this string, add "?debug" in the address line when you are using the page
#  "tableau.cgi", so the address should be something like: http://localhost/CCamelOT/tableau.cgi?debug
#  This will make the tableau show up the variable that each constraint returns.
#
#  In addition, the debug mode displays the mora and syllable arrays for each output.
#
#
#





#  faithfulness constraints

$CONdescription{F_Max}             = "No deletion of segments.     <P> Assign one violation mark for every segment   of the input  that is not present in the output.";
$CON{F_Max} = sub {
	return "M<SPAN STYLE='font-variant:small-caps;'>ax</SPAN>" unless (@_);      # return 0 (no violations) unless an input form is given to the function. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)
	my $outputForm = shift;         
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ;      # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for my $input (@{$inputForm->indices()}) {
		my $found = 0;
		for my $output (@{$outputForm->indices()}) {
			$found = 1 if ($input == $output);
		}
		$violations++ unless ($found); 
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{F_Maxmora}         = "No deletion of moras.        <P> Assign one violation mark for every mora   of the input  that is not present in the output. Note that this constraint also penalizes the deletion of a segment that heads a mora.";
$CON{F_Maxmora} = sub {
	return "M<SPAN STYLE='font-variant:small-caps;'>ax</SPAN>-" unless (@_);      # return 0 (no violations) unless an input form is given to the function. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)
	my $outputForm = shift;         
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for(@{$inputForm->moras()}) {
		my $currInputMoraIndex = (/^(\d+)_/)[0];
		my $found = 0;
		for (@{$outputForm->moras()}) {
			my $currOutputMoraIndex = (/^(\d+)_/)[0];
			$found = 1 if ($currInputMoraIndex == $currOutputMoraIndex);
		}
		$violations++ unless ($found); 
	}
	return $violations;
};  # don't delete this semicolon



$CONdescription{F_MaxC}            = "No deletion of consonants.   <P> Assign one violation mark for every consonant of the input  that is not present in the output.";
$CON{F_MaxC} = sub {
	return "M<SPAN STYLE='font-variant:small-caps;'>ax</SPAN>-C" unless (@_);      # return 0 (no violations) unless an input form is given to the function. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)
	my $outputForm = shift;         
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
		if (main::seg(${$inputForm->phonemes()}[$input])->cons() eq "C") {
			my $found = 0;
			for my $output (@{$outputForm->indices()}) {
				$found = 1 if (${$inputForm->indices()}[$input] == $output);
			}
			$violations++ unless ($found);
		}
	}
	return $violations;

};  # don't delete this semicolon

$CONdescription{F_MaxV}            = "No deletion of vowels.       <P> Assign one violation mark for every vowel     of the input  that is not present in the output.";
$CON{F_MaxV} = sub {
	return "M<SPAN STYLE='font-variant:small-caps;'>ax</SPAN>-V" unless (@_);      # return 0 (no violations) unless an input form is given to the function. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)
	my $outputForm = shift;         
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
		if (main::seg(${$inputForm->phonemes()}[$input])->cons() eq "V") {
			my $found = 0;
			for my $output (@{$outputForm->indices()}) {
				$found = 1 if (${$inputForm->indices()}[$input] == $output);
			}
			$violations++ unless ($found);
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{F_Dep}             = "No epenthesis of segments.   <P> Assign one violation mark for every segment   of the output that is not present in the input.";
$CON{F_Dep} = sub {
	return "D<SPAN STYLE='font-variant:small-caps;'>ep</SPAN>" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for my $output (@{$outputForm->indices()}) {
		my $found = 0;
		for my $input (@{$inputForm->indices()}) {
			$found = 1 if ($output == $input);
		}
		$violations++ unless ($found);
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{F_DepMora}         = "No epenthesis of moras.      <P> Assign one violation mark for every mora of the output that is not present in the input. Note that this constraint also penalizes the epenthesis of a segment that heads a mora.";
$CON{F_DepMora} = sub {
	return "D<SPAN STYLE='font-variant:small-caps;'>ep</SPAN>-" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for(@{$outputForm->moras()}) {
		my $currOutputMoraIndex = (/^(\d+)_/)[0];
		my $found = 0;
		for (@{$inputForm->moras()}) {
			my $currInputMoraIndex = (/^(\d+)_/)[0];
			$found = 1 if ($currInputMoraIndex == $currOutputMoraIndex);
		}
		$violations++ unless ($found); 
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{F_DepC}            = "No epenthesis of consonants. <P> Assign one violation mark for every consonant of the output that is not present in the input.";
$CON{F_DepC} = sub {
	return "D<SPAN STYLE='font-variant:small-caps;'>ep</SPAN>-C" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for (my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		if (main::seg(${$outputForm->phonemes()}[$output])->cons() eq "C") {
			my $found = 0;
			for my $input (@{$inputForm->indices()}) {
				$found = 1 if (${$outputForm->indices()}[$output] == $input);
			}
			$violations++ unless ($found);
		}
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{F_DepV}            = "No epenthesis of vowels.     <P> Assign one violation mark for every vowel     of the output that is not present in the input.";
$CON{F_DepV} = sub {
	return "D<SPAN STYLE='font-variant:small-caps;'>ep</SPAN>-V" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for (my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		if (main::seg(${$outputForm->phonemes()}[$output])->cons() eq "V") {
			my $found = 0;
			for my $input (@{$inputForm->indices()}) {
				$found = 1 if (${$outputForm->indices()}[$output] == $input);
			}
			$violations++ unless ($found);
		}
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{F_DepNonHi}            = "No epenthesis of non-high vowels.     <P> Assign one violation mark for every [&ndash;high] vowel     of the output that is not present in the input.";
$CON{F_DepNonHi} = sub {
	return "D<SPAN STYLE='font-variant:small-caps;'>ep</SPAN>-V(&ndash;hi)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for (my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		if (main::seg(${$outputForm->phonemes()}[$output])->cons() eq "V" and main::seg(${$outputForm->phonemes()}[$output])->high() eq "-") {
			my $found = 0;
			for my $input (@{$inputForm->indices()}) {
				$found = 1 if (${$outputForm->indices()}[$output] == $input);
			}
			$violations++ unless ($found);
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{F_IdentLength}     = "Segments must head the same number of moras in the input and in the output.<P> Assign one violation mark for every output segment that has an input correspondent, and the number of moras it heads in the input is different from the number of moras is heads in the output.";
$CON{F_IdentLength} = sub {

	return "I<SPAN STYLE='font-variant:small-caps;'>dent</SPAN>(length)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;

	for (my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
			if (${$outputForm->indices()}[$output] == ${$inputForm->indices()}[$input]) {
				$violations++ if ($inputForm->countMoras($input) != $outputForm->countMoras($output));
			}
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{F_IdentVoice}      = "Be faithful to [voice].                       <P> Assign one violation mark for every segment of the output that has an input correspondent, and the [voice] value of the output segment doesn't match the [voice] value of the input segment.";
$CON{F_IdentVoice} = sub {
	return "I<SPAN STYLE='font-variant:small-caps;'>dent</SPAN>(voice)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;

	for(my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
			if (${$outputForm->indices()}[$output] == ${$inputForm->indices()}[$input]) {
				$violations++ if (main::seg(${$outputForm->phonemes()}[$output])->voice() ne main::seg(${$inputForm->phonemes()}[$input])->voice()); 
			}
		}
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{F_IdentHigh}      = "Be faithful to [high].                       <P> Assign one violation mark for every segment of the output that has an input correspondent, and the [high] value of the output segment doesn't match the [high] value of the input segment.";
$CON{F_IdentHigh} = sub {
	return "I<SPAN STYLE='font-variant:small-caps;'>dent</SPAN>(high)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;

	for(my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
			if (${$outputForm->indices()}[$output] == ${$inputForm->indices()}[$input]) {
				$violations++ if (main::seg(${$outputForm->phonemes()}[$output])->high() ne main::seg(${$inputForm->phonemes()}[$input])->high()); 
			}
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{F_IdentLow}      = "Be faithful to [low].                       <P> Assign one violation mark for every segment of the output that has an input correspondent, and the [low] value of the output segment doesn't match the [low] value of the input segment.";
$CON{F_IdentLow} = sub {
	return "I<SPAN STYLE='font-variant:small-caps;'>dent</SPAN>(low)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;

	for(my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
			if (${$outputForm->indices()}[$output] == ${$inputForm->indices()}[$input]) {
				$violations++ if (main::seg(${$outputForm->phonemes()}[$output])->low() ne main::seg(${$inputForm->phonemes()}[$input])->low()); 
			}
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{F_IdentRootVoice}  = "Be faithful to [voice] in the root.           <P> Assign one violation mark for every segment of the output that has an input correspondent in the root, and the [voice] value of the output segment doesn't match the [voice] value of the input segment.";
$CON{F_IdentRootVoice} = sub {
	return "I<SPAN STYLE='font-variant:small-caps;'>dent<sub>root</sub></SPAN>(voice)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;

	for(my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		for (my $root=0 ; $root<@{$inputForm->root()} ; $root++) {
			if (${$outputForm->indices()}[$output] == ${$inputForm->root()}[$root]) {
				for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
					if (${$inputForm->root()}[$root] == ${$inputForm->indices()}[$input]) {
						$violations++ if (main::seg(${$outputForm->phonemes()}[$output])->voice() ne main::seg(${$inputForm->phonemes()}[$input])->voice());
					}
				}
			}
		}
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{F_IdentRootBack}   = "Be faithful to [back] in the root.            <P> Assign one violation mark for every segment of the output that has an input correspondent in the root, and the [back] value of the output segment doesn't match the [back] value of the input segment.";
$CON{F_IdentRootBack} = sub {
	return "I<SPAN STYLE='font-variant:small-caps;'>dent<sub>root</sub></SPAN>(back)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;

	for(my $output=0 ; $output<@{$outputForm->indices()} ; $output++) {
		for (my $root=0 ; $root<@{$inputForm->root()} ; $root++) {
			if (${$outputForm->indices()}[$output] == ${$inputForm->root()}[$root]) {
				for (my $input=0 ; $input<@{$inputForm->indices()} ; $input++) {
					if (${$inputForm->root()}[$root] == ${$inputForm->indices()}[$input]) {
						$violations++ if (main::seg(${$outputForm->phonemes()}[$output])->back() ne main::seg(${$inputForm->phonemes()}[$root])->back());
					}
				}
			}
		}
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{F_IdentOnsetVoice} = "Be faithful to [voice] of onset consonants.   <P> For every output segment that is in a syllable onset and has an input correspondent, assign one violation mark if the output and input correspondent don't have the same value for the feature [voice].";
$CON{F_IdentOnsetVoice} = sub {

	return "I<SPAN STYLE='font-variant:small-caps;'>dent<SUB>onset</SUB></SPAN>(voice)" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	for (@{$outputForm->syllables()}) {

		my $nucleusSonor = 0;
		my $nucleusLocation = 0;

		my @phonArr = $outputForm->findPhonemes($_);
		
		#find out the location of the nucleus
		for (my $i=0 ; $i<@phonArr ; $i++) {
			if (main::seg($phonArr[$i])->sonor() > $nucleusSonor) {
				$nucleusSonor = main::seg($phonArr[$i])->sonor();
				$nucleusLocation = $i
			}
		}
		
		# go over the onset consonants, and find out if their voicing is faithful
		if ($nucleusLocation>0) {
			# create an array that contains the indices of onset consonants
			my @currSyllIndices = split(/i/,(/^\d+\_si(.+)is$/)[0]);
			@currSyllIndices = @currSyllIndices[0..($nucleusLocation-1)];
			
			for (my $output=0 ; $output<@currSyllIndices ; $output++) {
				for (my $input=0; $input<@{$inputForm->indices()} ; $input++) {
					if ($currSyllIndices[$output] == ${$inputForm->indices()}[$input]) {
						my $voice = main::seg($phonArr[$output])->voice() . main::seg(${$inputForm->phonemes()}[$input])->voice();
						$violations++ if ($voice eq "-+" or $voice eq "+-");
					}
				}
			}
		}
	}
	return $violations;
};  # don't delete this semicolon



$CONdescription{F_OCONTIG}         = "No intrusion.                                 <P>Assign one violation mark for every pair of segments that are adjacent in the input and present in the output, but not adjacent in it.";
$CON{F_OCONTIG} = sub {
	return "O-C<SPAN STYLE='font-variant:small-caps;'>ontig</SPAN>" unless (@_);
	my $outputForm = shift; 
	my $inputForm  = shift ;
	return 0 unless ($inputForm) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)		

	my $violations = 0;
	
	my %outputPos;
	for (my $i=0 ; $i<@{$outputForm->indices()} ; $i++) {
		$outputPos{${$outputForm->indices()}[$i]} = $i;
	}
	
	for(my $i=0 ; $i<$#{$inputForm->indices()} ; $i++) {
	
		my $currentIndex = ${$inputForm->indices()}[$i];
		my $nextIndex    = ${$inputForm->indices()}[$i+1];
		if (exists($outputPos{$currentIndex}) and exists($outputPos{$nextIndex})) {
			$violations++ unless ($outputPos{$nextIndex} - $outputPos{$currentIndex} == 1); 
		}
	}
	return $violations;
};  # don't delete this semicolon



#  markedness constraints

# alignment

$CONdescription{M_AlignRRootSyll}  = "Roots must right-align with the right edge of a syllable. <P> For every root, Assign one violation mark if the rightmost segment of the root present in the output isn't the rightmost segment in some syllable.";
$CON{M_AlignRRootSyll} = sub {

	return "A<SPAN STYLE='font-variant:small-caps;'>lign</SPAN>R(root,&#963;)" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	#find last segment of root

	if (@{$outputForm->root()}) {
		# make a table of indices of root segments
		my %seen;
		$seen{$_} = 1 for (@{$outputForm->root()}) ;
	
		# use the table to make an array of output segments that are in the root
		my @outputRootSegment;
		for (@{$outputForm->indices()}) {
			push (@outputRootSegment,$_) if ($seen{$_}) ;
		}
	
		my $finalRootSegmentIndex = $outputRootSegment[$#outputRootSegment];
		my $isFinalinSyll = (index("@{$outputForm->syllables()}","i".$finalRootSegmentIndex."is")!=-1);
	
		return 1-$isFinalinSyll;
	} else {
		# no violation if no root
		return 0;
	}
};  # don't delete this semicolon


$CONdescription{M_AlignLRootWord}  = "Roots must left-align with the left edge of the Prosodic Word. <P> For every root, Assign one violation mark if the leftmost segment of the root present in the output isn't the leftmost segment in the Prosodic Word.";
$CON{M_AlignLRootWord} = sub {

	return "A<SPAN STYLE='font-variant:small-caps;'>lign</SPAN>L(root,PWd)" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	#find first segment of root

	if (@{$outputForm->root()}) {
		# make a table of indices of root segments
		my %seen;
		$seen{$_} = 1 for (@{$outputForm->root()}) ;

		if (exists $seen{${$outputForm->indices()}[0]}) {
			return 0 ;
		} else {
			return 1 ;
		}
	} else {
		# no violation if no root
		return 0;
	}
};  # don't delete this semicolon


$CONdescription{Mz_NoCoda}         = "Syllables must not have codas.                            <P> Assign one violation mark for every syllable whose final segment is a consonant, unless that segment is the most sonorous segment in the syllable (in which case it is the nucleus rather than the coda).";
$CON{Mz_NoCoda} = sub {

	return "*C<SPAN STYLE='font-variant:small-caps;'>oda</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my $maxSonor = 0;
		my @phonArr = $outputForm->findPhonemes($_);
		for (@phonArr) {
			$maxSonor = main::seg($_)->sonor() if (main::seg($_)->sonor() > $maxSonor);
		}

		my $lastSegSonor = main::seg($phonArr[$#phonArr])->sonor();
		my $lastSegCons  = main::seg($phonArr[$#phonArr])->cons();

		$violations++ if (($lastSegCons eq "C") and ($lastSegSonor<$maxSonor));
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{M_WBP}             = "Coda segments must head a mora (Weight By Position).      <P> Assign one violation mark for coda consonant that is not the most sonorous segment dominated by some mora.";
$CON{M_WBP} = sub {

	return "WBP" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my $maxSonor = 0;
		my $maxSonorLocation = 0;
		my @phonArr = $outputForm->findPhonemes($_);

		for (my $i=0 ; $i<@phonArr ; $i++) {
			if (main::seg($phonArr[$i])->sonor() > $maxSonor) {
				$maxSonor = main::seg($phonArr[$i])->sonor();
				$maxSonorLocation = $i;
			}
		}
		
		# find the position of the first segment in the syllable,
		# so moras of subsequent segments in the syllable can be counted relative to it
		my $firstSegIndex = (/^\d+_si(\d+)i/)[0];
		my $firstSegPos = 0;
		for (my $i=0 ; $i<@{$outputForm->indices()} ; $i++) {
			$firstSegPos = $i if (${$outputForm->indices()}[$i] == $firstSegIndex);
		}
		for (my $i = ($maxSonorLocation+1) ; $i < @phonArr ; $i++) {
			$violations++ if ($outputForm->countMoras($firstSegPos+$i)<1);
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{Mz_NoGlottalCoda}  = "Syllables must not end in a glottal stop.                 <P> Assign one violation mark for every syllable whose final segment is a glottal stop.";
$CON{Mz_NoGlottalCoda} = sub {

	return "*<SPAN CLASS=segment" . main::unicode() . ">" . main::seg("glot")->symbol() . "</SPAN>/<SMALL>C<SPAN STYLE='font-variant:small-caps;'>oda</SPAN></SMALL>" unless (@_) ;

	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;
	#my $test = "w";

	for (@{$outputForm->syllables()}) {
		my $maxSonor = 0;
		my @phonArr = $outputForm->findPhonemes($_);
		my $lastSegSonor = main::seg($phonArr[$#phonArr])->sonor();
		my $lastSegCons  = main::seg($phonArr[$#phonArr])->cons();
		
		for (@phonArr) {
			$maxSonor = main::seg($_)->sonor() if (main::seg($_)->sonor() > $maxSonor);
		}
		$violations++ if (($phonArr[$#phonArr] eq "glot") and ($lastSegCons eq "C") and ($lastSegSonor<$maxSonor));
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{Mz_NoOpenA}  = "Syllables must not end in a low vowel.                 <P> Assign one violation mark for every syllable whose final segment is a low vowel.";
$CON{Mz_NoOpenA} = sub {

	return "*a]" unless (@_) ;

	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my $maxSonor = 0;
		my @phonArr = $outputForm->findPhonemes($_);
		my $lastSegLow = main::seg($phonArr[$#phonArr])->low();
		my $lastSegCons  = main::seg($phonArr[$#phonArr])->cons();
		
		$violations++ if (($lastSegLow  eq "+") and ($lastSegCons eq "V"));
	}
	return $violations;
};  # don't delete this semicolon



$CONdescription{Mz_NoComplexCoda}  = "Syllables must not have codas made up of more than one segment. <P> Assign one violation mark for every syllable that ends in two consonants, neither of which is the most sonorous segment in the syllable (i.e. neither is the nucleus).";
$CON{Mz_NoComplexCoda} = sub {

	return "*C<SPAN STYLE='font-variant:small-caps;'>omplex</SPAN>C<SPAN STYLE='font-variant:small-caps;'>oda</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my @phonArr = $outputForm->findPhonemes($_);
		if (@phonArr>2) {
			my $maxSonor = 0;
			my $lastSegSonor = main::seg($phonArr[$#phonArr])->sonor();
			my $lastSegCons  = main::seg($phonArr[$#phonArr])->cons();
			my $penultSegSonor = main::seg($phonArr[$#phonArr-1])->sonor();
			my $penultSegCons  = main::seg($phonArr[$#phonArr-1])->cons();
			for (@phonArr) {
				$maxSonor = main::seg($_)->sonor() if (main::seg($_)->sonor() > $maxSonor);
			}
			$violations++ if (($lastSegCons eq "C") and ($penultSegCons eq "C") and ($penultSegSonor<$maxSonor) and ($lastSegSonor<$maxSonor));
		}
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{M_FinalC}          = "Words must end in a consonant.                            <P> Assign one violation mark for every word whose last segment is a vowel.";
$CON{M_FinalC} = sub {

	return "F<SPAN STYLE='font-variant:small-caps;'>inal</SPAN>-C" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	if (@{$outputForm->phonemes()}) {
		my $lastSegPhoneme = ${$outputForm->phonemes()}[$#{$outputForm->phonemes()}];
		$violations++ if (main::seg($lastSegPhoneme)->cons() eq "V");
	}

	return $violations;
};  # don't delete this semicolon


$CONdescription{Mz_NoCNucleus}     = "Syllables must not be headed by a consonant.              <P> Assign one violation mark for every syllable whose most sonorous segment is a consonant.";
$CON{Mz_NoCNucleus} = sub {

	return "*C/<SMALL>N<SPAN STYLE='font-variant:small-caps;'>uc</SPAN></SMALL>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my $maxSonor = 0;
		my @phonArr = $outputForm->findPhonemes($_); 
		for (@phonArr) {
			$maxSonor = main::seg($_)->sonor() if (main::seg($_)->sonor() > $maxSonor);
		}
		$violations++ if ($maxSonor < 12);
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{Mz_NoTrueGem}     = "There must not be true geminates.              <P> Assign one violation mark for every consonant that's associated with more than one sylllable.";
$CON{Mz_NoTrueGem} = sub {

	return "*T<SPAN STYLE='font-variant:small-caps;'>rue</SPAN>G<SPAN STYLE='font-variant:small-caps;'>em</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for my $index (@{$outputForm->indices()}) {
		my $found = 0;
		for (@{$outputForm->syllables()}) {
			$found++ if (/i${index}i/);
		}
		$violations++ if ($found>1);
	}
	return $violations;
};  # don't delete this semicolon



$CONdescription{Mz_NoCmora}        = "Moras must not be headed by a consonant.                  <P> Assign one violation mark for every mora whose most sonorous segment is a consonant.";
$CON{Mz_NoCmora} = sub {

	return "*/C" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for (@{$outputForm->moras()}) {
		my $maxSonor = 0;
		my @phonArr = $outputForm->findPhonemes($_); 
		for (@phonArr) {
			$maxSonor = main::seg($_)->sonor() if (main::seg($_)->sonor() > $maxSonor);
		}
		$violations++ if ($maxSonor < 12);
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{Mz_NoDiphthong}    = "No diphthongs.                                            <P> Assign one violation mark for every syllable whose nucleus is a vowel and the following segment in the syllable is also a vowel.";
$CON{Mz_NoDiphthong} = sub {

	return "*D<SPAN STYLE='font-variant:small-caps;'>iphthong</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form

	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my $nucleusSonor = 0;
		my $nucleusLocation = 0;
		my @phonArr = $outputForm->findPhonemes($_); 
		for (my $i=0; $i<@phonArr; $i++) {
			if (main::seg($phonArr[$i])->sonor() > $nucleusSonor) {
				$nucleusSonor = main::seg($phonArr[$i])->sonor();
				$nucleusLocation = $i;
			}
		}
		$violations++ if ($nucleusLocation < $#phonArr and main::seg($phonArr[$nucleusLocation])->cons eq "V" and main::seg($phonArr[$nucleusLocation+1])->cons eq "V");
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{M_Onset}           = "Syllables must have onsets.                               <P> Assign one violation mark for every syllable whose initial segment is the most sonorous segment in that syllable (i.e. it is the nucleus).";
$CON{M_Onset} = sub {

	return "O<SPAN STYLE='font-variant:small-caps;'>nset</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my $maxSonor = 0;
		my @phonArr = $outputForm->findPhonemes($_);
		for (@phonArr) {
			$maxSonor = main::seg($_)->sonor() if (main::seg($_)->sonor() > $maxSonor);
		}

		my $seg1sonor = main::seg($phonArr[0])->sonor();
		$violations++ unless (($seg1sonor < $maxSonor) and ($seg1sonor < 12));
	}
	return $violations;
};  # don't delete this semicolon



$CONdescription{Mz_NoAppendix}     = "Segments must belong to syllables.                        <P> Assign one violation mark for every segment that is not a part of some syllable.";
$CON{Mz_NoAppendix} = sub {

	return "*A<SPAN STYLE='font-variant:small-caps;'>ppendix</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	my $syllStr = "@{$outputForm->syllables()}";
	for (@{$outputForm->indices()}) {
		$violations++ if (index($syllStr,"i".$_."i")==-1);
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{M_CripsEdgeSyll}   = "Morpheme boundaries must be crisp.                        <P> Assign one violation mark for every morpheme boundary such that the segment before it and the segment after it belong to the same syllable.";
$CON{M_CripsEdgeSyll} = sub {

	return "C<SPAN STYLE='font-variant:small-caps;'>risp</SPAN>E<SPAN STYLE='font-variant:small-caps;'>dge</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	
	for (my $i=0 ; $i<$#{$outputForm->indices()} ; $i++) {

		my $seg0inRoot = 0;
		my $seg1inRoot = 0;
		for my $root (@{$outputForm->root()}) {
			$seg0inRoot = 1 if ($root == ${$outputForm->indices()}[$i]);
			$seg1inRoot = 1 if ($root == ${$outputForm->indices()}[$i+1]);
		}

		if ($seg0inRoot != $seg1inRoot) {
			for(@{$outputForm->syllables()}) {
				$violations++ if ((index($_,"i".${$outputForm->indices()}[$i]."i".${$outputForm->indices()}[$i+1]."i")!=-1));
			}
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{Mz_NoComplexOnset} = "Syllables must not have onsets made up of more than one segment.<P> Assign one violation mark for every syllable that starts with two consonants, neither of which is the most sonorous segment in the syllable (i.e. neither is the nucleus).";
$CON{Mz_NoComplexOnset} = sub {

	return "*C<SPAN STYLE='font-variant:small-caps;'>omplex</SPAN>O<SPAN STYLE='font-variant:small-caps;'>nset</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (@{$outputForm->syllables()}) {
		my $maxSonor = 0;
		my $seg1sonor = 0;
		my $seg2sonor = 0;

		my @phonArr = $outputForm->findPhonemes($_);
		if (@phonArr>2) {
			$seg1sonor = main::seg($phonArr[0])->sonor();
			$seg2sonor = main::seg($phonArr[1])->sonor();
			for (@phonArr) {
				$maxSonor = main::seg($_)->sonor() if (main::seg($_)->sonor() > $maxSonor);
			}
			$violations++ if (($seg1sonor < $maxSonor) and ($seg2sonor < $maxSonor) and ($seg1sonor < 12));
		}
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{Mz_No3Moras}       = "Syllables must not contain more than two moras.           <P> Starting from the most sonorous segment in each syllable (i.e. ignoring the onset), add up the moras headed by the segments in the syllable. Assign one violation mark for every syllable whose non-onset segments head more than two moras.";
$CON{Mz_No3Moras} = sub {

	return "*3" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (@{$outputForm->syllables()}) {


		# find where in the syllable the nucleus is
		my @phonArr  = $outputForm->findPhonemes($_);
		my $maxSonor = 0;
		my $maxSonorPos = 0;
		for (my $j=0 ; $j<@phonArr ; $j++) {
			if (main::seg($phonArr[$j])->sonor() > $maxSonor) {
				$maxSonor = main::seg($phonArr[$j])->sonor() ;
				$maxSonorPos = $j;
			}
		}

		# start counting moras only from the nucleus on
		my $moras = 0;
		my @indexArr = split(/i/,(/^\d+_si(.+)is$/)[0]);
		for (my $j=$maxSonorPos ; $j <@indexArr; $j++) {
			my $currPos = 0;
			for (my $k=0 ; $k<@{$outputForm->indices()} ; $k++) {
				$currPos = $k if (${$outputForm->indices()}[$k]  eq $indexArr[$j]) 
			}
			$moras += $outputForm->countMoras($currPos);
		}
		$violations++ if ($moras >2);
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{Mz_NoHeavy}       = "Syllables must not contain more than one mora.           <P> Starting from the most sonorous segment in each syllable (i.e. ignoring the onset), add up the moras headed by the segments in the syllable. Assign one violation mark for every syllable whose non-onset segments head more than one mora.";
$CON{Mz_NoHeavy} = sub {

	return "*H<SPAN STYLE='font-variant:small-caps;'>eavy</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (@{$outputForm->syllables()}) {


		# find where in the syllable the nucleus is
		my @phonArr  = $outputForm->findPhonemes($_);
		my $maxSonor = 0;
		my $maxSonorPos = 0;
		for (my $j=0 ; $j<@phonArr ; $j++) {
			if (main::seg($phonArr[$j])->sonor() > $maxSonor) {
				$maxSonor = main::seg($phonArr[$j])->sonor() ;
				$maxSonorPos = $j;
			}
		}

		# start counting moras only from the nucleus on
		my $moras = 0;
		my @indexArr = split(/i/,(/^\d+_si(.+)is$/)[0]);
		for (my $j=$maxSonorPos ; $j <@indexArr; $j++) {
			my $currPos = 0;
			for (my $k=0 ; $k<@{$outputForm->indices()} ; $k++) {
				$currPos = $k if (${$outputForm->indices()}[$k]  eq $indexArr[$j]) 
			}
			$moras += $outputForm->countMoras($currPos);
		}
		$violations++ if ($moras >1);
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{M_WordBin}         = "A word must contain two moras.                            <P> Add up the moras of the word that dominate at least one segment (i.e. ignore floating moras). Assign one violation mark for every word that has less than two such moras.";
$CON{M_WordBin} = sub {

	return "W<SPAN STYLE='font-variant:small-caps;'>ord</SPAN>B<SPAN STYLE='font-variant:small-caps;'>in</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	my $moras = 0;
	for my $currMora (@{$outputForm->moras()}) {
		$moras++ unless ($currMora =~ /_mm/); # only count moras that dominate segments
	}
	$violations = 1 if ($moras<2);

	return $violations;
};  # don't delete this semicolon


$CONdescription{Mz_NoVgV}          = "There must not be an intervocalic [g].                    <P> Assign one violation mark for every intervocalic [g].";
$CON{Mz_NoVgV} = sub {

	return "*VgV" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (my $i=0; $i<@{$outputForm->phonemes()}-2 ; $i++) {
		$violations++ if ((main::seg(${$outputForm->phonemes()}[$i])->cons() eq "V") and (${$outputForm->phonemes()}[$i+1] eq "g") and (main::seg(${$outputForm->phonemes()}[$i+2])->cons() eq "V"));
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{Mz_NoFakeGem}          = "There must not be fake geminates.                    <P> Assign one violation mark for pair of adjacent identical consonants.";
$CON{Mz_NoFakeGem} = sub {

	return "*F<SPAN STYLE='font-variant:small-caps;'>ake</SPAN>G<SPAN STYLE='font-variant:small-caps;'>em</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (my $i=0; $i<@{$outputForm->phonemes()}-1 ; $i++) {
		$violations++ if ((main::seg(${$outputForm->phonemes()}[$i])->cons() eq "C") and (main::seg(${$outputForm->phonemes()}[$i+1])->cons() eq "C") and (${$outputForm->phonemes()}[$i] eq ${$outputForm->phonemes()}[$i+1]) );
	}
	return $violations;
};  # don't delete this semicolon



$CONdescription{Mz_NoVTV}          = "There must not be an intervocalic voiceless stop.         <P> Assign one violation mark for every stop (i.e. [&ndash;cont, &ndash;son]) that is [&ndash;voice].";
$CON{Mz_NoVTV} = sub {

	return "*VTV" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (my $i=0; $i<@{$outputForm->phonemes()}-2 ; $i++) {
		$violations++ if ((main::seg(${$outputForm->phonemes()}[$i])->cons() eq "V") and (main::seg(${$outputForm->phonemes()}[$i+1])->son() eq "-") and (main::seg(${$outputForm->phonemes()}[$i+1])->cont() eq "-") and (main::seg(${$outputForm->phonemes()}[$i+1])->voice() eq "-") and (main::seg(${$outputForm->phonemes()}[$i+2])->cons() eq "V"));
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{M_AgreeVoice}      = "Adjacent obstruents must agree in [voice].                <P> Assign one violation mark for every pair of adjacent output obstruents that have different values for [voice].";
$CON{M_AgreeVoice} = sub {

	return "A<SPAN STYLE='font-variant:small-caps;'>gree</SPAN>(voice)" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (my $i=0; $i<@{$outputForm->phonemes()}-1 ; $i++) {
		$violations++ if ((main::seg(${$outputForm->phonemes()}[$i])->son() eq "-") and (main::seg(${$outputForm->phonemes()}[$i+1])->son() eq "-") and (main::seg(${$outputForm->phonemes()}[$i])->voice() eq main::seg(${$outputForm->phonemes()}[$i+1])->voice()));
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{M_AgreeBack}       = "Adjacent vowels must agree in [back] (consonants may intervene).<P> Assign one violation mark for every pair of output vowels that don't have another vowel between them, and have different values for [back].";
$CON{M_AgreeBack} = sub {

	return "A<SPAN STYLE='font-variant:small-caps;'>gree</SPAN>(back)" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	my $lastVowel = "";
	for (my $i=0; $i<@{$outputForm->phonemes()} ; $i++) {
		if (main::seg(${$outputForm->phonemes()}[$i])->cons() eq "V") {
			if ($lastVowel) {
				$violations++ if (main::seg($lastVowel)->back() ne main::seg(${$outputForm->phonemes()}[$i])->back());
			}
			$lastVowel = ${$outputForm->phonemes()}[$i];
		}
	}
	return $violations;
};  # don't delete this semicolon


$CONdescription{Mz_NoVoice}        = "No voiced obstruents.                                     <P> Assign one violation mark for every segment that is [&ndash;son] and [+voice].";
$CON{Mz_NoVoice} = sub {

	return "*V<SPAN STYLE='font-variant:small-caps;'>oice</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (my $i=0; $i<@{$outputForm->phonemes()} ; $i++) {
		$violations++ if ((main::seg(${$outputForm->phonemes()}[$i])->son() eq "-") and (main::seg(${$outputForm->phonemes()}[$i])->voice() eq "+"));
	}
	return $violations;
};  # don't delete this semicolon

$CONdescription{Mz_NoMid}        = "No mid vowels.                                     <P> Assign one violation mark for every vowel that is [&ndash;high] and [&ndash;low].";
$CON{Mz_NoMid} = sub {

	return "*M<SPAN STYLE='font-variant:small-caps;'>id</SPAN>" unless (@_);
	my $outputForm = shift;        ### don't read any more of @_, no need for input form
	my $violations = 0;

	for (my $i=0; $i<@{$outputForm->phonemes()} ; $i++) {
		$violations++ if ((main::seg(${$outputForm->phonemes()}[$i])->cons() eq "V") and (main::seg(${$outputForm->phonemes()}[$i])->high() eq "-") and (main::seg(${$outputForm->phonemes()}[$i])->low() eq "-"));
	}
	return $violations;
};  # don't delete this semicolon


# PREC Constraints


$CONdescription{P_LowEpen} = "Epenthesis must not feed a change of [low].<P> For every instance of epenthesis, assign one violation mark if a change of [low] crucially follows, and one violation mark if a change of [low] does not crucially precede.";
$CON{P_LowEpen} = sub {

	my $A = "Change low";
	my $B = "Epenthesis";

	return "P<SPAN STYLE='font-variant:small-caps;'>rec</SPAN>(low,epen)" unless (@_);
	my $currChain = shift ;
	return 0 unless ($currChain) ; # return 0 (no violations) unless an input form is given to the subroutine. This is what faithfulness constraints should do when there is no input (e.g. in syllabification)

	my @l_set   = @{$currChain->LUMs()};
	my @rLUMseq = @{$currChain->rLUMseq()};
	my $violations = 0;

	for my $Bprime (@l_set) {
		if ($Bprime =~ /$B/) {
			
			my $foundAB = 0;
			my $foundBA = 0;
			for (@rLUMseq) {
				$foundBA = 1 if (/$Bprime.+$A/);
				$foundAB = 1 if (/$A.+$Bprime/);
			}
			$violations++ if $foundBA;
			$violations++ unless $foundAB;
		}
	}

	return $violations;
};  # don't delete this semicolon






1